from typing import List

from dialog_bot_sdk.entities.Peer import PeerType, Peer
from dialog_bot_sdk.entities.UUID import UUID
from dialog_bot_sdk.entities.message.DocumentMessage import DocumentMessage
from dialog_bot_sdk.entities.message.ServiceMessage import ServiceMessage
from dialog_bot_sdk.entities.message.TextMessage import TextMessage
from dialog_api import messaging_pb2


class DeletedMessage:
    def __init__(self, is_local: bool) -> None:
        self.is_local = is_local

    @classmethod
    def from_api(cls, deleted: messaging_pb2.DeletedMessage) -> 'DeletedMessage':
        return cls(deleted.is_local)

    def __dict__(self):
        return {"is_local": self.is_local}

    def __str__(self):
        return "DeleteMessage({})".format(self.__dict__())


class JsonMessage:
    def __init__(self, raw_json: str) -> None:
        self.raw_json = raw_json

    @classmethod
    def from_api(cls, json_: messaging_pb2.JsonMessage) -> 'JsonMessage':
        return cls(json_.raw_json)

    def __dict__(self):
        return {"raw_json": self.raw_json}

    def __str__(self):
        return "JsonMessage({})".format(self.__dict__())


class MessageContent:
    def __init__(self, deleted: DeletedMessage, document: DocumentMessage, json_: JsonMessage,
                 service: ServiceMessage, text: TextMessage) -> None:
        self.deleted_message = deleted
        self.document_message = document
        self.json_message = json_
        self.service_message = service
        self.text_message = text

    @classmethod
    def from_api(cls, content: messaging_pb2.MessageContent) -> 'MessageContent':
        return cls(DeletedMessage.from_api(content.deletedMessage), DocumentMessage.from_api(content.documentMessage),
                   JsonMessage.from_api(content.jsonMessage), ServiceMessage.from_api(content.serviceMessage),
                   TextMessage.from_api(content.textMessage))

    def __dict__(self):
        return {"deleted_message": self.deleted_message.__dict__(), "document_message": self.document_message.__dict__(),
                "json_message": self.json_message.__dict__(),
                "service_message": self.service_message.__dict__(), "text_message": self.text_message.__dict__()}

    def __str__(self):
        return "MessageContent({})".format(self.__dict__())


class Message:
    def __init__(self, mid: UUID, prev_mid: UUID, sender_peer: Peer, message: MessageContent, reply: List[UUID],
                 forward: List[UUID], date: int, edited_at: int) -> None:
        self.mid = mid
        self.prev_mid = prev_mid
        self.sender_peer = sender_peer
        self.message = message
        self.reply = reply
        self.forward = forward
        self.date = date
        self.edited_at = edited_at

    @classmethod
    def from_api(cls, message: messaging_pb2.HistoryMessage) -> 'Message':
        return cls(UUID.from_api(message.mid), UUID.from_api(message.prev_mid),
                   Peer(message.sender_peer.id, PeerType.PEERTYPE_PRIVATE),
                   MessageContent.from_api(message.message), [UUID.from_api(x) for x in message.reply.mids],
                   [UUID.from_api(x) for x in message.forward.mids], message.date, message.edited_at.value)

    def __dict__(self):
        return {"mid": self.mid.__str__(), "prev_mid": self.prev_mid.__str__(),
                "sender_peer": self.sender_peer.__dict__(),
                "message": self.message.__dict__(), "reply": [x.__str__() for x in self.reply],
                "forward": [x.__str__() for x in self.forward], "date": self.date, "edited_at": self.edited_at}

    def __str__(self):
        return "Message({})".format(self.__dict__())



from dialog_bot_sdk.utils import async_dec
from .service import ManagedService
from dialog_api import sequence_and_updates_pb2


class Updates(ManagedService):
    """Class for handling grpc's server updates.

    """
    @async_dec()
    def get_difference(self, seq):
        """Raw implementation of API schema GetDifference method, which returns updates between attribute 'seq' and
        current seq at server side.

        :param seq: seq value
        :return: GetDifferenceResponse object
        """
        return self.internal.updates.GetDifference(sequence_and_updates_pb2.RequestGetDifference(seq=seq))

    @async_dec()
    def get_state(self):
        """Current application seq number

        :return: seq
        """
        return self.internal.updates.GetState(sequence_and_updates_pb2.RequestGetState()).seq



from dialog_api import media_and_files_pb2, messaging_pb2

from dialog_bot_sdk.entities.media.FileLocation import FileLocation
from dialog_bot_sdk.utils import AsyncTask


class ImageLocation:
    def __init__(self, file_location: FileLocation or AsyncTask, width: int = 100, height: int = 100,
                 file_size: int = 0) -> None:
        self.file_location = file_location
        self.width = width
        self.height = height
        self.file_size = file_size

    def to_api(self) -> media_and_files_pb2.ImageLocation:
        return media_and_files_pb2.ImageLocation(file_location=self.file_location.to_api(), width=self.width,
                                                 height=self.height, file_size=self.file_size)

    @classmethod
    def from_api(cls, image: media_and_files_pb2.ImageLocation) -> 'ImageLocation':
        return cls(FileLocation.from_api(image.file_location), image.width, image.height, image.file_size)

    def __dict__(self):
        return {"file_location": self.file_location.__dict__(), "width": self.width, "height": self.height,
                "file_size": self.file_size}

    def __str__(self):
        return "ImageLocation({})".format(self.__dict__())


class ImageMedia:
    def __init__(self, image: ImageLocation) -> None:
        self.image = image

    def to_api(self) -> messaging_pb2.ImageMedia:
        return messaging_pb2.ImageMedia(image=self.image.to_api())

    @classmethod
    def from_api(cls, image: messaging_pb2.ImageMedia) -> 'ImageMedia':
        return cls(ImageLocation.from_api(image))

    def __dict__(self):
        return {"image": self.image.__dict__()}

    def __str__(self):
        return "ImageMedia({})".format(self.__dict__())



from dialog_api import messaging_pb2

from dialog_bot_sdk.entities.Avatar import Avatar


class ServiceExt:
    def __init__(self, changed_about: str, changed_avatar: Avatar, changed_title: str,
                 changed_topic: str, user_invited: int, user_kicked: int) -> None:
        self.changed_about = changed_about
        self.changed_avatar = changed_avatar
        self.changed_title = changed_title
        self.changed_topic = changed_topic
        self.user_invited = user_invited
        self.user_kicked = user_kicked

    @classmethod
    def from_api(cls, service_ext: messaging_pb2.ServiceEx) -> 'ServiceExt':
        return cls(service_ext.changedAbout.about.value, Avatar.from_api(service_ext.changedAvatar.avatar),
                   service_ext.changedTitle.title, service_ext.changedTopic.topic.value,
                   service_ext.userInvited.invited_uid, service_ext.userKicked.kicked_uid)

    def __dict__(self):
        return {"changed_about": self.changed_about, "changed_avatar": self.changed_avatar.__dict__(),
                "changed_title": self.changed_title, "changed_topic": self.changed_topic,
                "user_invited": self.user_invited, "user_kicked": self.user_kicked}

    def __str__(self):
        return "ServiceExt({})".format(self.__dict__())


class ServiceMessage:
    def __init__(self, text: str, ext: ServiceExt) -> None:
        self.text = text
        self.ext = ext

    @classmethod
    def from_api(cls, service: messaging_pb2.ServiceMessage) -> 'ServiceMessage':
        return cls(service.text, ServiceExt.from_api(service.ext))

    def __dict__(self):
        return {"text": self.text, "ext": self.ext.__dict__()}

    def __str__(self):
        return "ServiceMessage({})".format(self.__dict__())



from typing import List

from dialog_bot_sdk.entities.FullUser import FullUser
from dialog_bot_sdk.entities.ReferencedEntities import ReferencedEntities
from dialog_bot_sdk.entities.User import User
from dialog_bot_sdk.entities.Peer import PeerType, Peer
from dialog_bot_sdk.utils import async_dec
from .service import ManagedService
from dialog_api import peers_pb2, users_pb2, messaging_pb2, search_pb2, sequence_and_updates_pb2


class Users(ManagedService):
    """Class for handling users
    """
    @async_dec()
    def get_user_by_nick(self, nick: str) -> User or None:
        """return User by nickname

        :param nick: user's nickname
        :return: User
        """
        if not isinstance(nick, str):
            raise RuntimeError("Invalid input data. Expects {}, got {}.".format(str, type(nick)))
        out_peer = self.__find_out_peer_by_nick(nick)
        if out_peer.id == out_peer.access_hash == 0:
            return None

        request = sequence_and_updates_pb2.RequestGetReferencedEntitites(
                users=[
                    peers_pb2.UserOutPeer(uid=out_peer.id, access_hash=out_peer.access_hash)
                ]
            )
        result = ReferencedEntities.from_api(self.internal.updates.GetReferencedEntitites(request))
        for user in result.users:
            if hasattr(user.data, "nick") and user.data.nick == nick:
                return user

    @async_dec()
    def get_user_by_id(self, user_id: int) -> User or None:
        """return User by user id

        :param user_id: user's id
        :return: User or None if not found
        """
        if not isinstance(user_id, int):
            raise RuntimeError("Invalid input data. Expects {}, got {}.".format(int, type(user_id)))
        req = messaging_pb2.RequestLoadDialogs(
            min_date=0,
            limit=1,
            peers_to_load=[peers_pb2.Peer(type=peers_pb2.PEERTYPE_PRIVATE, id=user_id)]
        )
        result = self.internal.messaging.LoadDialogs(req)
        users_list = self.internal.updates.GetReferencedEntitites(
            sequence_and_updates_pb2.RequestGetReferencedEntitites(
                users=list(result.user_peers)
            )
        )

        for user in users_list.users:
            if hasattr(user, "id") and user.id == user_id:
                return User.from_api(user)

    @async_dec()
    def search_users_by_nick_substring(self, query: str) -> List[User]:
        """Returns list of User objects by substring of nickname (not complete coincidence!)

        :param query: user's nickname
        :return: list of Users
        """
        request = search_pb2.RequestPeerSearch(
                query=[
                    search_pb2.SearchCondition(
                        searchPeerTypeCondition=search_pb2.SearchPeerTypeCondition(
                            peer_type=search_pb2.SEARCHPEERTYPE_CONTACTS)
                    ),
                    search_pb2.SearchCondition(
                        searchPieceText=search_pb2.SearchPieceText(query=query)
                    )
                ]
            )
        user_peers = self.internal.search.PeerSearch(request).user_peers
        request = sequence_and_updates_pb2.RequestGetReferencedEntitites(
                users=list(user_peers)
            )
        users_list = ReferencedEntities.from_api(self.internal.updates.GetReferencedEntitites(request)).users
        result = []

        for user in users_list:
            if hasattr(user.data, "nick") and query in user.data.nick:
                result.append(user)
        return result

    @async_dec()
    def get_full_profile_by_nick(self, nick: str) -> FullUser or None:
        """return FullUser profile by nickname

        :param nick: user's nickname
        :return: FullUser
        """
        out_peer = self.__find_out_peer_by_nick(nick)
        return self.get_full_profile_by_id(out_peer.id).wait()

    @async_dec()
    def get_full_profile_by_id(self, id_: int) -> FullUser or None:
        out_peer = self.manager.get_out_peer(Peer(id_, PeerType.PEERTYPE_PRIVATE))
        if not out_peer:
            return None
        request = users_pb2.RequestLoadFullUsers(
            user_peers=[
                peers_pb2.UserOutPeer(
                    uid=out_peer.id,
                    access_hash=out_peer.access_hash
                )
            ]
        )
        full_profile = self.internal.users.LoadFullUsers(request)

        if full_profile and hasattr(full_profile, 'full_users') and len(full_profile.full_users) > 0:
            return FullUser.from_api(full_profile.full_users[0])

    def __find_out_peer_by_nick(self, nick: str) -> peers_pb2.OutPeer:
        if not isinstance(nick, str):
            raise RuntimeError("Invalid input data. Expects {}, got {}.".format(str, type(nick)))
        request = search_pb2.RequestResolvePeer(
                shortname=nick
            )
        result = self.internal.search.ResolvePeer(request)
        if hasattr(result, "peer"):
            self.manager.add_out_peer(result.peer)
            return result.peer



import unittest
from dialog_api import peers_pb2, messaging_pb2, definitions_pb2
from dialog_api.peers_pb2 import PEERTYPE_PRIVATE
from mock import patch

from dialog_bot_sdk.entities.ListLoadMode import ListLoadMode
from dialog_bot_sdk.entities.Peer import Peer
from dialog_bot_sdk.entities.UUID import UUID
from dialog_bot_sdk.entities.media.ImageMedia import ImageMedia, ImageLocation
from dialog_bot_sdk.entities.message.Message import Message, MessageContent
from dialog_bot_sdk.entities.message.TextMessage import MessageMedia
from dialog_bot_sdk.interactive_media import InteractiveMediaGroup, InteractiveMedia, InteractiveMediaButton
from dialog_bot_sdk.utils import AsyncTask
from tests.bot import bot
from tests.test_classes.media_and_files import MediaAndFiles, Put

from tests.test_classes.messaging import Messaging
from tests.test_classes.updates import Updates


class TestMessaging(unittest.TestCase):
    bot.internal.messaging = Messaging()
    bot.internal.updates = Updates()
    bot.internal.media_and_files = MediaAndFiles()

    test_file = "../dialog_bot_sdk/examples/files/example.png"

    peer = Peer(1, PEERTYPE_PRIVATE)
    message = Message(UUID(1, 1), UUID(0, 0), peer, None, [], [], 0, 0)
    outpeer = peers_pb2.OutPeer(type=PEERTYPE_PRIVATE, id=0, access_hash=0)
    mid = definitions_pb2.UUIDValue(msb=0, lsb=0)
    msg_content = messaging_pb2.MessageContent()
    msg_content.textMessage.text = "Hello"
    interactive_media = [InteractiveMediaGroup(
        [
            InteractiveMedia(
                "1",
                InteractiveMediaButton("Yes", "Да")
            ),
            InteractiveMedia(
                "2",
                InteractiveMediaButton("No", "Нет")
            ),
        ]
    )]
    # msg_content_with_group = copy.deepcopy(msg_content)
    # group[0].render(msg_content_with_group.textMessage.media.add())
    # doc_msg = messaging_pb2.DocumentMessage(
    #     file_id=0,
    #     access_hash=0,
    #     file_size=60,
    #     name=""
    # )

    def test_send_message(self):
        send = bot.messaging.send_message(self.peer, "text", self.interactive_media)
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertTrue(isinstance(send.wait(), UUID))

    def test_update_message(self):
        update = bot.messaging.update_message(self.message, "text", self.interactive_media)
        self.assertTrue(isinstance(update, AsyncTask))
        self.assertIsNone(update.wait())

    def test_delete_message(self):
        delete = bot.messaging.delete(self.message)
        self.assertTrue(isinstance(delete, AsyncTask))
        self.assertIsNone(delete.wait())

    def test_get_messages_by_id(self):
        msg = bot.messaging.get_messages_by_id([UUID(1, 1)])
        self.assertTrue(isinstance(msg, AsyncTask))
        self.assertTrue(isinstance(msg.wait()[0], Message))

    def test_messages_read(self):
        read = bot.messaging.messages_read(self.peer, 0)
        self.assertTrue(isinstance(read, AsyncTask))
        self.assertIsNone(read.wait())

    @patch('requests.put')
    def test_send_file(self, put):
        put.return_value = Put(200)
        send = bot.messaging.send_file(self.peer, self.test_file)
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertTrue(isinstance(send.wait(), UUID))
        put.return_value = Put(400)
        send = bot.messaging.send_file(self.peer, self.test_file)
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertIsNone(send.wait())

    @patch('requests.put')
    def test_send_media(self, put):
        put.return_value = Put(200)
        image = bot.internal.uploading.upload_file(self.test_file).wait()
        media = MessageMedia(image=ImageMedia(ImageLocation(image)))
        send = bot.messaging.send_media(self.peer, [media])
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertTrue(isinstance(send.wait(), UUID))

    @patch('requests.put')
    def test_send_image(self, put):
        put.return_value = Put(200)
        send = bot.messaging.send_image(self.peer, self.test_file)
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertTrue(isinstance(send.wait(), UUID))
        put.return_value = Put(400)
        send = bot.messaging.send_image(self.peer, self.test_file)
        self.assertTrue(isinstance(send, AsyncTask))
        self.assertIsNone(send.wait())

    def test_reply(self):
        reply = bot.messaging.reply(self.peer, [UUID(1, 1)], None, self.interactive_media)
        self.assertTrue(isinstance(reply, AsyncTask))
        self.assertTrue(isinstance(reply.wait(), UUID))

    def test_forward(self):
        forward = bot.messaging.forward(self.peer, [UUID(1, 1)], None, self.interactive_media)
        self.assertTrue(isinstance(forward, AsyncTask))
        self.assertTrue(isinstance(forward.wait(), UUID))

    def test_load_message_history(self):
        history = bot.messaging.load_message_history(self.peer, 0, ListLoadMode.LISTLOADMODE_BACKWARD, 1)
        self.assertTrue(isinstance(history, AsyncTask))
        self.assertTrue(isinstance(history.wait()[0], Message))


if __name__ == '__main__':
    unittest.main()



from typing import List

from dialog_api import messaging_pb2
from dialog_api.messaging_pb2 import INTERACTIVEMEDIASTYLE_UNKNOWN, INTERACTIVEMEDIASTYLE_DEFAULT, \
    INTERACTIVEMEDIASTYLE_PRIMARY, INTERACTIVEMEDIASTYLE_DANGER
from google.protobuf.wrappers_pb2 import StringValue


class InteractiveMediaStyle:
    INTERACTIVEMEDIASTYLE_UNKNOWN = INTERACTIVEMEDIASTYLE_UNKNOWN
    INTERACTIVEMEDIASTYLE_DEFAULT = INTERACTIVEMEDIASTYLE_DEFAULT
    INTERACTIVEMEDIASTYLE_PRIMARY = INTERACTIVEMEDIASTYLE_PRIMARY
    INTERACTIVEMEDIASTYLE_DANGER = INTERACTIVEMEDIASTYLE_DANGER


class InteractiveMediaButton:
    def __init__(self, value: str, label: str) -> None:
        self.value = value
        self.label = label

    def to_api(self) -> messaging_pb2.InteractiveMediaButton:
        return messaging_pb2.InteractiveMediaButton(value=StringValue(value=self.value),
                                                    label=StringValue(value=self.label))

    @classmethod
    def from_api(cls, button: messaging_pb2.InteractiveMediaButton) -> 'InteractiveMediaButton':
        return cls(button.value.value, button.label)

    def __dict__(self):
        return {"value": self.value, "label": self.label}

    def __str__(self):
        return "InteractiveMediaButton({})".format(self.__dict__())


class InteractiveMediaSelectOption:
    def __init__(self, value: str, label: str) -> None:
        self.value = value
        self.label = label

    def to_api(self) -> messaging_pb2.InteractiveMediaSelectOption:
        return messaging_pb2.InteractiveMediaSelectOption(value=StringValue(value=self.value),
                                                          label=StringValue(value=self.label))

    @classmethod
    def from_api(cls, option: messaging_pb2.InteractiveMediaSelectOption) -> 'InteractiveMediaSelectOption':
        return cls(option.value.value, option.label.value)

    def __dict__(self):
        return {"value": self.value, "label": self.label}

    def __str__(self):
        return "InteractiveMediaSelectOption({})".format(self.__dict__())


class InteractiveMediaSelect:
    def __init__(self, options: List[InteractiveMediaSelectOption], label: str, default_value: str):
        self.options = options
        self.label = label
        self.default_value = default_value

    def to_api(self) -> messaging_pb2.InteractiveMediaSelect:
        return messaging_pb2.InteractiveMediaSelect(options=[x.to_api() for x in self.options],
                                                    default_value=StringValue(value=self.default_value),
                                                    label=StringValue(value=self.label))

    @classmethod
    def from_api(cls, select: messaging_pb2.InteractiveMediaSelect) -> 'InteractiveMediaSelect':
        return cls([InteractiveMediaSelectOption.from_api(x) for x in select.options], select.lable.value,
                   select.default_value.value)

    def __dict__(self):
        return {"options": [x.__dict__() for x in self.options], "label": self.label,
                "default_value": self.default_value}

    def __str__(self):
        return "InteractiveMediaSelect({})".format(self.__dict__())


class InteractiveMediaWidget:
    def __init__(self, interactive_media_button: InteractiveMediaButton,
                 interactive_media_select: InteractiveMediaSelect) -> None:
        self.interactive_media_button = interactive_media_button
        self.interactive_media_select = interactive_media_select

    def to_api(self) -> messaging_pb2.InteractiveMediaWidget:
        return messaging_pb2.InteractiveMediaWidget(interactiveMediaButton=self.interactive_media_button.to_api(),
                                                    interactiveMediaSelect=self.interactive_media_select.to_api())

    @classmethod
    def from_api(cls, widget: messaging_pb2.InteractiveMediaWidget) -> 'InteractiveMediaWidget':
        return cls(InteractiveMediaButton.from_api(widget.interactiveMediaButton),
                   InteractiveMediaSelect.from_api(widget.interactiveMediaSelect))

    def __dict__(self):
        return {"interactive_media_button": self.interactive_media_button.__dict__(),
                "interactive_media_select": self.interactive_media_select.__dict__()}

    def __str__(self):
        return "InteractiveMediaWidget({})".format(self.__dict__())


class InteractiveMediaConfirm:
    def __init__(self, text: str, title: str, ok: str, dismiss: str):
        self.text = text
        self.title = title
        self.ok = ok
        self.dismiss = dismiss

    def to_api(self) -> messaging_pb2.InteractiveMediaConfirm:
        return messaging_pb2.InteractiveMediaConfirm(text=StringValue(value=self.text),
                                                     title=StringValue(value=self.title),
                                                     ok=StringValue(value=self.ok),
                                                     dismiss=StringValue(value=self.dismiss))

    @classmethod
    def from_api(cls, confirm: messaging_pb2.InteractiveMediaConfirm) -> 'InteractiveMediaConfirm':
        return cls(confirm.text.value, confirm.title.value, confirm.ok.value, confirm.dismiss.value)

    def __dict__(self):
        return {"text": self.text, "title": self.title, "ok": self.ok, "dismiss": self.dismiss}

    def __str__(self):
        return "InteractiveMediaConfirm({})".format(self.__dict__())


class InteractiveMedia:
    def __init__(self, id: str, widget: InteractiveMediaWidget, style: InteractiveMediaStyle,
                 confirm: InteractiveMediaConfirm) -> None:
        self.id = id
        self.widget = widget
        self.style = style
        self.confirm = confirm

    def to_api(self) -> messaging_pb2.InteractiveMedia:
        return messaging_pb2.InteractiveMedia(id=self.id, widget=self.widget,
                                              style=self.style,
                                              confirm=self.confirm.to_api())

    @classmethod
    def from_api(cls, media: messaging_pb2.InteractiveMedia) -> 'InteractiveMedia':
        return cls(media.id, InteractiveMediaWidget.from_api(media.widget),
                   media.style, InteractiveMediaConfirm.from_api(media.confirm))

    def __dict__(self):
        return {"id": self.id, "widget": self.widget.__dict__(), "style": self.style, "confirm": self.confirm.__dict__()}

    def __str__(self):
        return "InteractiveMedia({})".format(self.__dict__())


class InteractiveMediaTranslation:
    def __init__(self, id: str, value: str):
        self.id = id
        self.value = value

    def to_api(self) -> messaging_pb2.InteractiveMediaTranslation:
        return messaging_pb2.InteractiveMediaTranslation(id=StringValue(value=self.id),
                                                         value=StringValue(value=self.value))

    @classmethod
    def from_api(cls, translation: messaging_pb2.InteractiveMediaTranslation) -> 'InteractiveMediaTranslation':
        return cls(translation.id.value, translation.value.value)

    def __dict__(self):
        return {"id": self.id, "value": self.value}

    def __str__(self):
        return "InteractiveMediaTranslation({})".format(self.__dict__())


class InteractiveMediaTranslationGroup:
    def __init__(self, language: str, messages: List[InteractiveMediaTranslation]):
        self.language = language
        self.messages = messages

    def to_api(self) -> messaging_pb2.InteractiveMediaTranslationGroup:
        return messaging_pb2.InteractiveMediaTranslationGroup(language=StringValue(value=self.language),
                                                              messages=[x.to_api() for x in self.messages])

    @classmethod
    def from_api(cls, translation: messaging_pb2.InteractiveMediaTranslationGroup) -> 'InteractiveMediaTranslationGroup':
        return cls(translation.language.value, [InteractiveMediaTranslation.from_api(x) for x in translation.messages])

    def __dict__(self):
        return {"language": self.language, "messages": [x.__dict__() for x in self.messages]}

    def __str__(self):
        return "InteractiveMediaTranslationGroup({})".format(self.__dict__())


class InteractiveMediaGroup:
    def __init__(self, actions: List[InteractiveMedia], title: str, description: str,
                 translations: List[InteractiveMediaTranslationGroup]) -> None:
        self.actions = actions
        self.title = title
        self.description = description
        self.translations = translations

    def to_api(self) -> messaging_pb2.InteractiveMediaGroup:
        return messaging_pb2.InteractiveMediaGroup(actions=[x.to_api() for x in self.actions], title=self.title,
                                                   description=self.description,
                                                   translations=[x.to_api() for x in self.translations])

    @classmethod
    def from_api(cls, media_group: messaging_pb2.InteractiveMediaGroup) -> 'InteractiveMediaGroup':
        return cls([InteractiveMedia.from_api(x) for x in media_group.actions], media_group.title,
                   media_group.description,
                   [InteractiveMediaTranslationGroup.from_api(x) for x in media_group.translations])

    def __dict__(self):
        return {"actions": [x.__dict__() for x in self.actions], "title": self.title, "description": self.description,
                "translations": [x.__dict__() for x in self.translations]}

    def __str__(self):
        return "InteractiveMediaGroup({})".format(self.__dict__())



from dialog_api import peers_pb2
from dialog_api.peers_pb2 import PEERTYPE_PRIVATE, PEERTYPE_GROUP, PEERTYPE_UNKNOWN, PEERTYPE_ENCRYPTEDPRIVATE, \
    PEERTYPE_SIP


class PeerType:
    PEERTYPE_UNKNOWN = PEERTYPE_UNKNOWN
    PEERTYPE_PRIVATE = PEERTYPE_PRIVATE
    PEERTYPE_GROUP = PEERTYPE_GROUP
    PEERTYPE_ENCRYPTEDPRIVATE = PEERTYPE_ENCRYPTEDPRIVATE
    PEERTYPE_SIP = PEERTYPE_SIP


class Peer:
    def __init__(self, id: int, type: PeerType) -> None:
        self.id = id
        self.type = type

    def to_api(self) -> peers_pb2.Peer:
        return peers_pb2.Peer(id=self.id, type=self.type)

    @classmethod
    def from_api(cls, peer: peers_pb2.Peer) -> 'Peer':
        return cls(peer.id, peer.type)

    def __dict__(self):
        return {"id": self.id, "type": self.type}

    def __str__(self):
        return "Peer({})".format(self.__dict__())

    def __eq__(self, other):
        if self.id == other.id and self.type == other.type:
            return True
        return False



from dialog_api import media_and_files_pb2, messaging_pb2

from dialog_bot_sdk.entities.media.FileLocation import FileLocation
from dialog_bot_sdk.utils import AsyncTask


class AudioLocation:
    def __init__(self, file_location: FileLocation or AsyncTask, duration: int = 0, mime_type: str = "",
                 file_size: int = 0) -> None:
        self.file_location = file_location
        self.duration = duration
        self.mime_type = mime_type
        self.file_size = file_size

    def to_api(self) -> media_and_files_pb2.AudioLocation:
        return media_and_files_pb2.AudioLocation(file_location=self.file_location.to_api(), duration=self.duration,
                                                 mime_type=self.mime_type, file_size=self.file_size)

    @classmethod
    def from_api(cls, audio: media_and_files_pb2.AudioLocation) -> 'AudioLocation':
        return cls(FileLocation.from_api(audio.file_location), audio.duration, audio.mime_type.value, audio.file_size)

    def __dict__(self):
        return {"file_location": self.file_location.__dict__(), "duration": self.duration,
                "mime_type": self.mime_type, "file_size": self.file_size}

    def __str__(self):
        return "AudioLocation({})".format(self.__dict__())


class AudioMedia:
    def __init__(self, audio: AudioLocation) -> None:
        self.audio = audio

    def to_api(self) -> messaging_pb2.AudioMedia:
        return messaging_pb2.AudioMedia(audio=self.audio.to_api())

    @classmethod
    def from_api(cls, image: messaging_pb2.AudioMedia) -> 'AudioMedia':
        return cls(AudioLocation.from_api(image))

    def __dict__(self):
        return {"audio": self.audio.__dict__()}

    def __str__(self):
        return "AudioMedia({})".format(self.__dict__())



from dialog_api import users_pb2
from dialog_bot_sdk.entities.Avatar import Avatar
from dialog_bot_sdk.entities.Peer import PeerType, Peer


class UserData:
    def __init__(self, name: str, avatar: Avatar, about: str, nick: str) -> None:
        self.name = name
        self.avatar = avatar
        self.about = about
        self.nick = nick

    @classmethod
    def from_api(cls, data: users_pb2.UserData) -> 'UserData':
        if data:
            return cls(data.name, Avatar.from_api(data.avatar), data.about.value, data.nick.value)

    def __dict__(self):
        return {"name": self.name, "avatar": self.avatar.__dict__(), "about": self.about, "nick": self.nick}

    def __str__(self):
        return "{}".format(self.__dict__())


class User:
    def __init__(self, id: int, data: UserData = None) -> None:
        self.data = data
        self.peer = Peer(id, PeerType.PEERTYPE_PRIVATE)

    @classmethod
    def from_api(cls, user: users_pb2.User) -> 'User':
        return cls(user.id, UserData.from_api(user.data))

    def __dict__(self):
        return {"data": self.data.__dict__(), "peer": self.peer.__dict__()}

    def __str__(self):
        return "User({})".format(self.__dict__())



from .service import AuthenticatedService
from dialog_api import registration_pb2, registration_pb2_grpc, \
    sequence_and_updates_pb2_grpc, \
    authentication_pb2, authentication_pb2_grpc, \
    contacts_pb2_grpc, search_pb2_grpc, messaging_pb2_grpc, \
    media_and_files_pb2_grpc, users_pb2_grpc, groups_pb2_grpc
from dialog_bot_sdk.uploading import Uploading


class InternalBot(object):
    """Class with Dialog bot's internal services.

    """

    def __init__(self, channel, verbose=False, cert=None, private_key=None, access_dir=None, options=None):
        self.app_id = 10
        self.app_title = "PythonBotSDK/1.0"
        self.channel = channel
        self.registration = self.wrap_service(registration_pb2_grpc.RegistrationStub, verbose=verbose, options=options)
        self.messaging = self.wrap_service(messaging_pb2_grpc.MessagingStub, verbose=verbose, options=options)
        self.media_and_files = self.wrap_service(media_and_files_pb2_grpc.MediaAndFilesStub, verbose=verbose, options=options)
        self.updates = self.wrap_service(sequence_and_updates_pb2_grpc.SequenceAndUpdatesStub, verbose=verbose, options=options)
        self.auth = self.wrap_service(authentication_pb2_grpc.AuthenticationStub, verbose=verbose, options=options)
        self.contacts = self.wrap_service(contacts_pb2_grpc.ContactsStub, verbose=verbose, options=options)
        self.search = self.wrap_service(search_pb2_grpc.SearchStub, verbose=verbose, options=options)
        self.users = self.wrap_service(users_pb2_grpc.UsersStub, verbose=verbose, options=options)
        self.groups = self.wrap_service(groups_pb2_grpc.GroupsStub, verbose=verbose, options=options)
        self.token = self.get_session_token()
        self.uploading = Uploading(self, cert, private_key, access_dir=access_dir)

    def authorize(self, bot_token):
        """Authorization function for Internal bot instance.

        :param bot_token: bot token
        :return: auth token (instance of gRPC RequestStartTokenAuth)
        """
        return self.auth.StartTokenAuth(authentication_pb2.RequestStartTokenAuth(
            token=bot_token,
            app_id=self.app_id
        ))

    def anonymous_authorize(self, name='dumb'):
        return self.auth.StartAnonymousAuth(authentication_pb2.RequestStartAnonymousAuth(
            name=name
        ))

    def get_session_token(self):
        """Requests for sessions token for device.

        :return: session token
        """
        registration_response = self.registration.RegisterDevice(
            registration_pb2.RequestRegisterDevice(
                app_id=self.app_id,
                app_title=self.app_title,
                device_title=self.app_title
            )
        )
        return registration_response.token

    def wrap_service(self, stub_func, verbose=False, options=None):
        """Wrapper for authenticating of gRPC service calls.

        :param options: retries options
        :param stub_func: name of gRPC service
        :param verbose: verbosity level of functions calling
        :return: wrapped gRPC service
        """
        return AuthenticatedService(
            lambda: self.token if hasattr(self, 'token') else None,
            stub_func(self.channel),
            verbose=verbose,
            options=options
        )
